// TODO: loose: all?
// TODO: external helpers?

var babel = require('babel-core')
	, extend = require('lodash/object/extend')

	, BOILERPLATE = 'var global = window.global = window;\n\n'
	, DEFAULT_HELPERS = [
			'class-call-check',
			'inherits',
			'tagged-template-literal-loose'
		]
	, HELPERS = babel.buildExternalHelpers(DEFAULT_HELPERS)
	, SETTINGS = {
			externalHelpers: true,
			loose: 'all',
			whitelist: [
				'es6.arrowFunctions',
				'es6.blockScoping',
				'es6.classes',
				'es6.constants',
				'es6.destructuring',
				'es6.parameters',
				'es6.properties.computed',
				'es6.properties.shorthand',
				'es6.spread',
				// 'es6.tailCall',
				'es6.templateLiterals',
				'es6.regex.sticky',
				'es6.regex.unicode',
				'react',
				'react.displayName'/*,
				'optimisation.react.constantElements',
				'optimisation.react.inlineElements',
				'minification.deadCodeElimination'*/
			]
		};

/**
 * Retrieve registration data
 */
exports.registration = {
	name: 'babel',
	extensions: {
		js: [
			'es6',
			'es7',
			'js',
			'jsx'
		]
	}
};

/**
 * Compile 'content'
 * @param {String} content
 * @param {Object} options
 * @param {Function} fn(err, content)
 */
exports.compile = function (content, options, fn) {
	// Skip node_modules files
	if (!~options.filepath.indexOf('node_modules')) {
		try {
			var transform = babel.transform(content, extend({}, SETTINGS))
				, helpers = (transform.metadata.usedHelpers || []).filter(function (helper) {
						return !~DEFAULT_HELPERS.indexOf(helper);
					});

			if (helpers.length) {
				// TODO: add missing helpers to content
			}

			// Store helper boilerplate
			if (~transform.code.indexOf('babelHelpers')) {
				options.cache.setSource('js-helpers', BOILERPLATE + HELPERS);
			}

			fn(null, transform.code);
		} catch (err) {
			err.filepath = options.filepath;
			fn(err);
		}
	} else {
		fn(null, content);
	}
};
